{% extends 'base.html' %}

{% block content %}
	<script type="text/javascript">
		var PipelineSettings = {
			'apiUrl': '/api/status/',
			'detailUrlPrefix': '/detail/',
			'remainingSecondsWarning': 5 * 60,
			'remainingSecondsDanger': 10 * 60
		}
	</script>
	<script type="text/jsx">{% include 'components.jsx' %}</script>
	<script type="text/jsx">{% include 'pipeline.jsx' %}</script>
{% endblock %}